import re

def parse_results(soup):
    results = []
    for series in soup.find_all('div',attrs={'class':'col-12 col-lg-6 p-3 text'}):
        box = series.div.find_all('div',recursive=False)

        info = box[1].div.find_all('div',recursive=False)

        ratings_text = re.search(r'(?P<year>\d{4})*(?: - )*(?:(?P<rating>\d+\.(\d+)) \/ 10\.0)*',info[3].get_text())

        thumbnail = box[0].find('img')
        is_adult = False
        if not thumbnail:
            is_adult = bool(box[0].find(lambda tag: tag.get_text() == 'AdultContent'))
        else:
            thumbnail = thumbnail['src']
            # when a manga is considered to contain 'Adult Content', mangaupdates replaces the thumbnail with a divsoup that renders to a centered 'Adult Content' on a white background.

        results.append({
            'name': info[0].a.u.b.get_text(),
            'id': info[0].a['href'].replace('https://www.mangaupdates.com/series.html?id=',''),
            'genres': info[1].a['title'].split(', '),
            'summary': info[2].get_text(),
            'year': ratings_text.group('year'),
            'rating': ratings_text.group('rating'),
            'thumbnail': thumbnail,
            'is_adult': is_adult
        })
    return results



def parse_releases(list):
    releases = list.find_all('div',class_='text')[:-1]
    results = []

    for i in range(0,len(releases),5):
        release = {}

        release['date'] = str(releases[i].string)

        series_link = releases[i+1]
        if series_link.a is None:
            release['name'] = str(series_link.string)
        else:
            release['name'] = str(series_link.a.string)
            release['id'] = series_link.a['href'].replace('https://www.mangaupdates.com/series.html?id=','')

        release['vol'] = str(releases[i+2].string)
        release['chp'] = str(releases[i+3].string)
        release['group'] = str(releases[i+4].string)

        results.append(release)

    return results

def parse_series(list):
    series = list.find_all('div',class_='text')[:-1]
    results = []

    for i in range(0,len(series),4):
        manga = {}

        series_link = series[i]
        if series_link.a is None:
            manga['name'] = str(series_link.string)
        else:
            manga['name'] = str(series_link.a.string)
            manga['id'] = series_link.a['href'].replace('https://www.mangaupdates.com/series.html?id=','')

        manga['genres'] = str(series[i+1].a.string).replace(' ','').split(',')
        manga['year'] = str(series[i+2].string)
        manga['rating'] = str(series[i+3].string)

        results.append(manga)

    return results

def parse_scanlators(list):
    scanlators = list.find_all('div',class_='text')[:-1]
    results = []

    for i in range(0,len(scanlators),3):
        group = {}

        group_link = scanlators[i]
        if group_link.a is None:
            group['name'] = str(group_link.string)
        else:
            group['name'] = str(group_link.a.string)
            group['id'] = group_link.a['href'].replace('https://www.mangaupdates.com/groups.html?id=','')

        group['active'] = str(scanlators[i+1].string)

        group_contact = scanlators[i+2]
        if group_contact.a is None:
            group['contact'] = str(group_contact.string)
        else:
            group['contact'] = group_contact.a['href']

        results.append(group)

    return results

def parse_authors(list):
    authors = list.find_all('div',class_='text')[:-1]
    results = []

    for i in range(0,len(authors),3):
        author = {}

        author_link = authors[i]
        if author_link.a is None:
            author['name'] = str(author_link.string)
        else:
            author['name'] = str(author_link.a.string)
            author['id'] = author_link.a['href'].replace('https://www.mangaupdates.com/authors.html?id=','')

        author['series'] = str(authors[i+1].string)

        author['genres'] = []

        for gen in authors[i+2].find_all('a'):
            author['genres'].append(str(gen.string))

        results.append(author)

    return results




#unfinished - small tool to automatically notify you of the latest manga releases
from pynotifier import Notification
import json
import api
from config_path import ConfigPath
conf_path = ConfigPath('alisw','pymanga','.json')
path = conf_path.readFolderPath()
path.mkdir(parents=True,exist_ok=True)
config_path = path.joinpath('config.json')

if not config_path.exists():
    with open(config_path,'w') as f:
        f.write(json.dumps({
            'ids': [],
            'date_limit': 7
        }))

config = {}
with open(config_path,'r') as f:
    config = json.loads(f.read())

for id in config['ids']:
    series = api.series(id)
    latest = series['latest_releases'][0]
    if int(latest['date'].replace(' days ago','')) < config['date_limit']:
        Notification(
            title='New ' + series['title'] + ' Chapter!',
            description='Chapter ' + latest['chapter'] + ' released.',
            duration=15,
            urgency=Notification.URGENCY_NORMAL
        ).send()



import markdownify
from bs4 import Comment

def parse_series(content):
    manga = {}
    manga['title'] = str(content.find('span',class_='releasestitle').string)

    cols = content.find_all('div',class_='col-6',recursive=False)
    col_1 = cols[0]
    col_2 = cols[1]

    parse_col_1(col_1,manga)
    parse_col_2(col_2,manga)
    return manga

def parse_col_1(col,manga):
    contents = col.find_all('div',class_='sContent',recursive=False)

    desc_tag = contents[0]
    for c in desc_tag.findAll(text=lambda text:isinstance(text, Comment)):
        c.extract()

    manga['description'] = markdownify.markdownify(desc_tag.encode_contents())
    manga['type'] = str(contents[1].string).replace('\n','')
    manga['related_series'] = str(contents[2].string).replace('\n','')
    manga['associated_names'] = str(contents[3].string).replace('\n','')
    if contents[4].a is None:
        manga['group'] = {'name': str(contents[4].string)}
    else:
        manga['group'] = {'name': str(contents[4].a.string), 'id': contents[4].a.get('href','').replace('https://www.mangaupdates.com/groups.html?id=','')}

    manga['latest_releases'] = []
    numbers = contents[5].find_all('i')[:-1]
    groups = contents[5].find_all('a')[:-1]
    dates = contents[5].find_all('span')

    for i in range(0,len(dates)):
        release = {
            'group': {
                'name': str(groups[i].string),
                'id': str(groups[i].get('href','').replace('https://www.mangaupdates.com/groups.html?id=',''))
            },
            'date': dates[i]['title']
        }

        # this is to check if there are volume numbers. its a bad solution, folks!
        if len(numbers) >= len(dates) * 2:
            release['volume'] = str(numbers[i].string)
            release['chapter'] = str(numbers[i+1].string)
        else:
            release['chapter'] =  str(numbers[i].string)

        manga['latest_releases'].append(release)


    manga['status'] = str(contents[6].string).replace('\n','')
    if str(contents[7].string).replace('\n','') == 'No':
        manga['completely_scanlated'] = False
    else:
        manga['completely_scanlated'] = True
    manga['anime_chapters'] = str(contents[8].string).replace('\n','')
    manga['user_reviews'] = str(contents[9].string).replace('\n','')

    manga['forum'] = {
        'status': str(contents[10].string),
        'link': 'https://www.mangaupdates.com/' + contents[10].a.get('href','')
    }

    try:
        average_raw = contents[11].contents
        manga['average'] = {
            'average': str(average_raw[0]).replace('Average:','').replace(' ',''),
            'votes': str(average_raw[2]).replace('(','').replace(')',''),
            'bayesian': str(average_raw[5]).replace('<b>','').replace('</b>','')
        }
    except:
        manga['average'] = {}

    manga['last_updated'] = str(contents[12].string).replace('\n','')

def parse_col_2(col,manga):
    contents = col.find_all('div',class_='sContent',recursive=False)

    manga['image'] = contents[0].center.img['src']

    manga['genres'] = []
    for genre in contents[1].find_all('a')[:-1]:
        manga['genres'].append(str(genre.u.string))

    manga['categories'] = []
    if contents[2].div:
        for cat_raw in contents[2].div.ul.find_all('li'):
            cat = cat_raw.find('a',rel='nofollow')
            manga['categories'].append({
                'category': str(cat.string),
                'score': str(cat['title']).replace('Score:','')
            })

    manga['category_recs'] = []
    for rec in contents[3].find_all('a'):
        manga['category_recs'].append({
            'name': str(rec.get_text()),
            'id': rec.get('href','').replace('series.html?id=','')
        })

    manga['recs'] = str(contents[4].get_text()).replace('\n','')

    manga['authors'] = []
    for author in contents[5].find_all('a'):
        manga['authors'].append({
            'name': str(author.get_text()),
            'id': author.get('href','').replace('https://www.mangaupdates.com/authors.html?id=','')  if contents[5].a else 'N/A'
        })

    manga['artists'] = []
    for artist in contents[6].find_all('a'):
        manga['artists'].append({
            'name': str(artist.get_text()),
            'id': artist.get('href','').replace('https://www.mangaupdates.com/authors.html?id=','')  if contents[6].a else 'N/A'
        })

    manga['year'] = str(contents[7].get_text()).replace('\n','')

    manga['publisher'] = {
        'name': str(contents[8].get_text()),
        'id': contents[8].a.get('href','').replace('https://www.mangaupdates.com/publishers.html?id=','')  if contents[8].a else 'N/A'
    }

    # TODO: add publisher info
    manga['serialized'] = {
        'name': str(contents[9].get_text()),
        'link': 'https://www.mangaupdates.com/' + contents[9].a.get('href','') if contents[9].a else ''
    }

    manga['licensed'] = str(contents[10].string).replace('\n','')

    # TODO: add volume/ongoing info
    manga['english_publisher'] = {
        'name': str(contents[11].get_text()),
        'id': str(contents[11].a.get('href','').replace('https://www.mangaupdates.com/publishers.html?id=','') if contents[11].a else 'N/A')
    }

    pos_r = contents[12].contents

    manga['positions'] = {
        'weekly': str(pos_r[2].string),
        'weekly_change': str(pos_r[5].string).replace('(','').replace(')',''),
        'monthly': str(pos_r[9].string),
        'monthly_change': str(pos_r[12].string).replace('(','').replace(')',''),
        'tri_monthly': str(pos_r[16].string),
        'tri_monthly_change': str(pos_r[19].string).replace('(','').replace(')',''),
        'six_monthly': str(pos_r[23].string),
        'six_monthly_change': str(pos_r[26].string).replace('(','').replace(')',''),
        'yearly': str(pos_r[30].string),
        'yearly_change': str(pos_r[33].string).replace('(','').replace(')','')
    }

    read_lists = contents[13].find_all('b')
    manga['reading_lists'] = {
        'reading': str(read_lists[0].get_text()),
        'wish': str(read_lists[1].get_text()),
        'unfinished': str(read_lists[2].get_text()),
        'custom': str(read_lists[3].get_text())
    }



import requests
from bs4 import BeautifulSoup
import urllib.parse as urlparse
import re
import traceback
import os
from .parsers import search_parsers, series_parsers, releases_parsers, adv_search_parser

def search(query):
    r = requests.post('https://mangaupdates.com/search.html',params={'search':query})
    print(r.request.body)
    soup = BeautifulSoup(r.text,'html.parser')
    lists = soup.find('div',class_='center-side-bar').find_all('div',class_='row',recursive=False)[1].find('div',id='main_content').find_all('div',class_='row')

    results = {}
    try:
        results['releases'] = search_parsers.parse_releases(lists[0])
    except:
        results['releases'] = []
    try:
        results['series'] = search_parsers.parse_series(lists[1])
    except:
        traceback.print_exc()
        results['series'] = []
    try:
        results['scanlators'] = search_parsers.parse_scanlators(lists[2])
    except:
        results['scanlators'] = []
    try:
        results['authors'] = search_parsers.parse_authors(lists[3])
    except:
        results['authors'] = []
    return results

# Warning: this can take a long time on general searches with the 'all_pages' option enabled!
def advanced_search(params,all_pages=False,page=1):
    params['page'] = page
    params['perpage'] = 100

    # do some quality of life processing (turn list args. into properly escaped parameters)
    params['genre'] = '_'.join(params.get('genre',[]))
    params['exclude_genre'] = '_'.join(params.get('exclude_genre',[]))
    params['category'] ='_'.join(params.get('category',[]))

    if 'name' in params:
        params['search'] = params['name']

    payload = urlparse.urlencode(params,safe='+')

    r = requests.get('https://mangaupdates.com/series.html',params=payload)
    soup = BeautifulSoup(r.text,'html.parser')
    # find page count. i know regex is a crime but.

    pages = re.search(r'Pages \((\d+)\)',r.text)
    if pages:
        pages = int(pages.group(1))
    else:
        pages = 1

    results = adv_search_parser.parse_results(soup)

    # if all pages is set, request every page and append their results to the return value

    if all_pages and pages > 1:
        for p in range(page+1,pages+1):
            r, _ = advanced_search(params,page=p)
            results = results + r

        os.sleep(1) # be polite!

    return (results,pages)

# advanced search, but with a generator!
def advanced_search_iter(params):
    first_batch, total_pages = advanced_search(params)
    yield from first_batch

    for p in range(2,total_pages+1):
        r, _ = advanced_search(params,page=p)
        yield from r

def series(id):
    r = requests.get('https://mangaupdates.com/series.html',params={'id': id})
    soup = BeautifulSoup(r.text,'html.parser')
    content = soup.find('div',class_='center-side-bar').find_all('div',class_='row',recursive=False)[1].find('div',id='main_content').find('div',class_='p-2',recursive=False).find('div',class_='row',recursive=False)
    series = {}
    try:
        series = series_parsers.parse_series(content)
    except:
        traceback.print_exc()
        series = {}
    return series

def releases(id):
    r = requests.post('https://www.mangaupdates.com/releases.html',params={'stype': 'series','search': id,'page':1},data={'perpage':100})
    soup = BeautifulSoup(r.text,'html.parser')
    content = soup.find('div',class_='center-side-bar').find_all('div',class_='row',recursive=False)[1].find('div',id='main_content').find('div',class_='p-2',recursive=False).find('div',class_='row',recursive=False)
    releases = []
    try:
        releases = releases_parsers.parse_releases(content)
    except:
        releases = []
    return releases



def parse_releases(content):
    releases = content.find_all('div',class_='text',recursive=False)[:-1]
    results = []

    for i in range(0,len(releases),5):
        release = {}
        release['series'] = {
            'name': str(releases[i+1].a.string),
            'id': releases[i+1].a['href'].replace('https://www.mangaupdates.com/series.html?id=','')
        }
        release['vol'] = str(releases[i+2].string)
        release['chapter'] = str(releases[i+3].string)
        release['group'] = {
            'name': str(releases[i+4].a.string),
            'id': releases[i+4].a['href'].replace('https://www.mangaupdates.com/groups.html?id=','')
        }
        results.append(release)

    return results
